'use strict';
/*global XR */

/**
 * Created by x.Laoxu on 5/1/16.
 */
/**
 * This mixin automatically sets the state for you based on the key you provide
 *
 * Usage:
 *
 * mixins: [ReactStateMagicMixin],
 *
 * statics: {
 *   registerStores: {
 *     foo: FooStore,
 *     bar: BarStore
 *   }
 * },
 *
 * render: function () {
 *   // state will be in the keys you provided
 *   this.state.foo
 *   this.state.bar
 * }
 *
 * Alternatively:
 *
 * statics: {
 *   registerStore: FooStore
 * },
 *
 * render: function () {
 *   // all of FooStore's state will be dumped into this.state
 *   this.state
 * }
 */

var _Subscribe = require('./Subscribe');

var _Subscribe2 = _interopRequireDefault(_Subscribe);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ReactStateMagicMixin = {
    getInitialState: function getInitialState() {
        return this.getStateFromStores();
    },

    componentDidMount: function componentDidMount() {
        _Subscribe2.default.create(this);

        if (this.constructor.registerStore && this.constructor.registerStores) {
            throw new ReferenceError('You are attempting to use `registerStore` and `registerStores` ' + 'pick one');
        }

        if (this.constructor.registerStore) {
            _Subscribe2.default.add(this, this.constructor.registerStore, this.altSetState);
        } else {
            var stores = this.constructor.registerStores;
            if (!stores) {
                return;
            }
            Object.keys(stores).forEach(function (formatter) {
                _Subscribe2.default.add(this, stores[formatter], this.altSetState);
            }, this);
        }
    },

    componentWillUnmount: function componentWillUnmount() {
        _Subscribe2.default.destroy(this);
    },

    getStateFromStores: function getStateFromStores() {
        if (!this.constructor.registerStore && !this.constructor.registerStores) {
            return null;
        }

        if (this.constructor.registerStore) {
            return this.constructor.registerStore.getState();
        }

        var stores = this.constructor.registerStores;

        return Object.keys(stores).reduce(function (obj, key) {
            return obj[key] = stores[key].getState(), obj;
        }, {});
    },

    altSetState: function altSetState() {
        this.setState(this.getStateFromStores());
    }
};

module.exports = ReactStateMagicMixin;