'use strict';

/**
 * Created by x.Laoxu on 12/18/15.
 */

var hasWebp = function (callback) {
    var handleOnLoad = function (support) {
        return function () {
            hasWebp = function (cb) {
                cb && cb(support);
                return support;
            };
            return hasWebp(callback);
        };
    };
    var img = new Image();
    img.onload = handleOnLoad(true);
    img.onerror = handleOnLoad(false);
    img.src = 'data:image/webp;base64,UklGRkoAAABXRUJQVlA4WAoAAAAQAAAAAAAAAAAAQUxQSAsAAAABBxAREYiI/gcAAABWUDggGAAAADABAJ0BKgEAAQABABwlpAADcAD+/gbQAA==';
    return null;
};


export let call = (func, ...args) => {
    // const Promise = require('bluebird');
    if (func && (typeof func === 'function')) {
        let r = func.call(func, ...args);
        if (r && r.then) {
            return r;
        }
        return Promise.resolve(r);
    }
    return Promise.resolve(false);
};

export let isWeiXin = () => {
    let result = /MicroMessenger/i.test(navigator.userAgent);
    isWeiXin = () => result;
    return isWeiXin();
};


const array = {
    fill: (length, f) => {
        let r = [];
        for (let x = 0; x < length; x++) {
            r.push(f);
        }
        return r;
    }
};

let OS = () => {
    let os = {}, ua = navigator.userAgent,
        android = ua.match(/(Android);?[\s\/]+([\d.]+)?/),
        iphone = !ipad && ua.match(/(iPhone\sOS)\s([\d_]+)/),
        ipod = ua.match(/(iPod)(.*OS\s([\d_]+))?/),
        ipad = ua.match(/(iPad).*OS\s([\d_]+)/);
    if (android) os.android = true, os.version = android[2]
    if (iphone && !ipod) os.ios = os.iphone = true, os.version = iphone[2].replace(/_/g, '.')
    if (ipad) os.ios = os.ipad = true, os.version = ipad[2].replace(/_/g, '.');

    OS = () => os;

    return OS();
};

const string = {
    jsonToQueryString: (json) => {
        if (!json) {
            return '';
        }
        return Object.keys(json).map(function (key) {
            return encodeURIComponent(key) + '=' +
                encodeURIComponent(json[key]);
        }).join('&');
    }
};

const client = {
    goto: (url) => {
        window.location = url;
    },
    setTitle: (title) => {
        window.document.title = title;
    },

    on: (node, event, listener) => {
        if (node.addEventListener) {
            node.addEventListener(event, listener, false);
        } else if (node.attachEvent) {
            node.attachEvent(event, listener);
        }
    },
    un: (node, event, listener) => {
        if (node.removeEventListener) {
            node.removeEventListener(event, listener, false);
        } else if (node.detachEvent) {
            node.detachEvent(event, listener);
        }
    }
};
const DATE_S = {
    'd': 60 * 60 * 24 * 1000,//day
    'm': (date, num) => {
        var newDate = new Date(date.getTime());
        newDate.setMonth(date.getMonth() + num);
        return newDate;
    }
};

const TIME_A = {
    'm': (num) => num * 60,
    's': (num) => num,
    'h': (num) => num * 60 * 60
};

const WEEK_NAME = ['日', '一', '二', '三', '四', '五', '六'];

const date = {
    removeSecondText: (timeText) => {
        return timeText.split(':').splice(0, 2).join(':');
    },
    /**
     * 字符 1:2:1,转为秒数:1*60*60 + 2*60 + 1
     * @param time
     * @returns seconds int
     */
    time2Seconds: (time) => {
        const l = [60 * 60, 60];
        return time.split(':')
            .map(t => parseInt(t) || 0)
            .concat([0])
            .splice(0, 3)
            .reduceRight((a, t, i) => {
                return a + (t * l[i]);
            })
    },
    second2Time: (seconds) => {
        const hFloat = seconds / 3600;
        const h = parseInt(hFloat);
        const s = parseInt((hFloat - h) * 60);
        return h + ':' + number.fill(s);
    },
    week: (day) => {
        let d = day;
        if (day instanceof Date) {
            d = day.getDay();
        }
        return WEEK_NAME[d];
    },
    new: (date) => {

        if (typeof date === 'string' && date.indexOf('+0800') > 0) {
            date = date.replace('+0800', '+08:00');
        }

        let d = (date instanceof Date) ? date : new Date(date);
        return d;
    },
    today: () => {
        const now = new Date();
        return new Date(now.getFullYear(), now.getMonth(), now.getDate());
    },

    addTime: function (time, num, s = 'm') {
        if (typeof time === 'string') {
            if (time.indexOf(':') > -1) {
                time = this.time2Seconds(time);
            }
            else {
                time = parseInt(time);
            }
        }
        const added_time = time + TIME_A[s](num);

        return this.second2Time(added_time);
    },

    add: function (date, num, s = 'd') {
        if (num === 0) {
            return date;
        }
        let d = this.new(date);//(date instanceof Date) ? date : new Date(date);

        if (typeof DATE_S[s] === 'function') {
            return DATE_S[s](date, num);
        }

        let now = (d.getTime()) + (num * DATE_S[s]);
        // console.log(date, new Date(now), num, s);
        return new Date(now);
    },
    formatTime: function (date) {
        let d = this.new(date);
        let hour = number.fill(d.getHours());
        let minute = number.fill(d.getMinutes());
        return `${hour}:${minute}`;
    },
    format: function (time, shortMonth = false, full = false) {

        let now = this.new(time);// (time instanceof Date) ? time : new Date(time);
        //let now         = new Date(time);
        let year = now.getFullYear();
        let month = now.getMonth() + 1;
        let date = now.getDate();

        if (shortMonth) {
            return number.fill(month) + '.' + number.fill(date);
        }

        const format = year + '-' + number.fill(month) + '-' + number.fill(date);

        if (full) {
            let hour = now.getHours();
            let minute = now.getMinutes();
            let second = now.getSeconds();
            // let millisecond = now.getMilliseconds();
            return format + ' ' + number.fill(hour) + ':' + number.fill(minute) + ':' + number.fill(second);
        }
        return format;

    },

    countdown: (starttime, endtime, format = true) => {

        let diffSeconds = (endtime - starttime) / 1000;
        let hours = '00',
            minutes = '00',
            seconds = '00',
            milliseconds = '00';


        if (diffSeconds <= 0) {
            return {hours, minutes, seconds, milliseconds, _empty: true};
        }

        let hoursFloat = diffSeconds / 3600;
        hours = parseInt(hoursFloat);

        let minuteFloat = (hoursFloat - hours) * 60;
        minutes = parseInt(minuteFloat);

        let secondFloat = (minuteFloat - minutes) * 60;
        seconds = parseInt(secondFloat);

        milliseconds = parseInt((secondFloat - seconds) * 1000);


        return format ? {
            hours: number.fill(hours, 2),
            minutes: number.fill(minutes, 2),
            seconds: number.fill(seconds, 2),
            milliseconds: number.fill(milliseconds, 3, '0', 2)
        } : {hours, minutes, seconds, milliseconds};
    }
};

const number = {
    fill: (number, slice = 2, fill = '0', beginSlice = 0) => {
        let fills = '';

        if (String(number).length < slice) {
            let fillLength = slice - String(number).length;
            for (let i = 0; i < fillLength; i++) {
                fills += '' + fill;
            }
        }
        fills += number;
        return beginSlice > 0 ? fills.slice(0, beginSlice) : fills.slice(-slice);
    }
};

const element = {
    hasClass: (ele, cls) => {
        return ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'));
    },
    addClass: (ele, cls) => {
        if (!element.hasClass(ele, cls)) ele.className += " " + cls;
    },
    removeClass: (ele, cls) => {
        if (element.hasClass(ele, cls)) {
            var reg = new RegExp('(\\s|^)' + cls + '(\\s|$)');
            ele.className = ele.className.replace(reg, ' ');
        }
    },
    toggleClass: (ele, cls) => {
        if (element.hasClass(ele, cls)) {
            element.removeClass(obj, cls);
        } else {
            element.addClass(ele, cls);
        }
    }
};


const Noop = function () {
    return false;
};


const stopEvent = (event) => {
    if (event && event.preventDefault) {
        event.preventDefault();
        event.stopPropagation();
    }
};


const objValue = (obj, keys, value, deep) => {

    if (typeof obj !== 'object' || typeof keys !== 'string') {
        console.log(obj, keys);
        throw new Error('Type of [obj] or [keys] is illegal');
    }

    let keyList = keys.split('.');
    let lastKey = keyList.pop();
    // let obj = obj;

    for (let i in keyList) {
        const p = keyList[i];
        if (typeof obj === 'object' && obj[p] === undefined) {
            if (deep) {
                obj[p] = {};
                obj = obj[p];
                continue;
            }
            return;
        }

        obj = obj[p];
    }

    let result = false;
    if (typeof obj === 'object') {
        obj[lastKey] = value;
        result = true;
    }
    // console.log(JSON.stringify(order));
    return result;
};

const extendObject = (target, ...sourceObj) => {
    sourceObj.map(s => {
        for (let o in s) {
            target[o] = s[o];
        }
    });
};


const X = {
    Noop,
    stopEvent,
    hasWebp,
    call,
    isWeiXin,
    window: client,
    date,
    number,
    string,
    element,
    array,
    objValue,
    extendObject,
    OS
};
//
//(()=> {
//    X.window.on(window, 'hashchange', (r)=> {
//        console.log(r);
//    });
//})();


export default X;
